
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>

// digit pattern for a 7-segment display
const uint8_t digit_pattern[16] =
	{
		0b00111111, // 0
		0b00000110, // 1
		0b01011011, // 2
		0b01001111, // 3
		0b01100110, // 4
		0b01101101, // 5
		0b01111101, // 6
		0b00000111, // 7
		0b01111111, // 8
		0b01101111, // 9
		0b01110111, // A
		0b01111100, // b
		0b00111001, // C
		0b01011110, // d
		0b01111001, // E
		0b01110001  // F
};

volatile uint8_t counter = 0;
volatile uint8_t enabled = 0;

/* intreruperea de comparatie cu OCR1A */
ISR(TIMER1_COMPA_vect)
{
	if (enabled)
	{
		PORTD ^= (1 << PD2);
		counter += 1;
		if (counter > 99)
		{
			counter = 0;
		}
	}
}

/* intreruperea de tip pin change pe portul D */
ISR(PCINT3_vect)
{
	if ((PIND & (1 << PD3)) == 0)
	{
		PORTD ^= (1 << PD4);
		enabled = !enabled;
	}
}

/* initializeaza intrarile si iesirile */
void IO_init()
{
	DDRD &= ~(1 << PD3);
	PORTD |= (1 << PD3);

	DDRD |= (1 << PD4);
	PORTD &= ~(1 << PD4);
	DDRD |= (1 << PD2);
	PORTD &= ~(1 << PD2);

	// drive segments
	DDRA |= (1 << PA0);
	DDRA |= (1 << PA1);
	DDRA |= (1 << PA2);
	DDRA |= (1 << PA3);
	DDRA |= (1 << PA4);
	DDRA |= (1 << PA5);
	DDRA |= (1 << PA6);

	// activate digit
	DDRA |= (1 << PA7);
	DDRC |= (1 << PC2);

	// Activez PIN CHANGE INTERRUPT ENABLE si PCIE3 pentru PCINT24:31 (portul D)
	PCICR |= (1 << PCIE3);
	// Activez intreruperea pe butonul PD3
	PCMSK3 |= (1 << PCINT27);
}

/* initializeaza timer-ul 1 */
void timer1_init()
{
	TCNT1 = 0;
	// f_int = 1hz
	// OCR1A = 62500;
	OCR1A = 46875;
	// Activeaza intreruperea de comparare cu OCR1A
	TIMSK1 = (1 << OCIE1A);
	// CTC cu top la OCR1A
	TCCR1B = (1 << WGM12);
	// Prescaler 256
	TCCR1B |= (1 << CS12);
}

void output_7segment(uint8_t val);

uint8_t digits[2];

/* get number digits */
void get_digits(uint8_t num, uint8_t ndigits)
{
	uint8_t dig = 0;

	for (uint8_t i = 0; i < ndigits; i++)
	{
		digits[dig] = num % 10;
		num = num / 10;
		dig++;
	}
}

void set_digit(uint8_t dig)
{
	if (dig == 0)
	{
		PORTC &= ~(1 << PC2);
		PORTA |= (1 << PA7);
	}
	else
	{
		PORTA &= ~(1 << PA7);
		PORTC |= (1 << PC2);
	}
}

void output_7segment(uint8_t val)
{
	// first clear the port for a new val
	// mask last bit as it's used for other stuff
	PORTA &= (1 << PA7);
	// update the bits without changing PA7
	PORTA |= digit_pattern[val] & (~(1 << PA7));
}

int main(void)
{

	IO_init();

	_delay_ms(1000);

	timer1_init();
	sei();

	PORTD |= (1 << PD4);
	_delay_ms(500);
	PORTD &= ~(1 << PD4);
	_delay_ms(500);

	while (1)
	{
		get_digits(counter, 2);
		set_digit(1);
		output_7segment(digits[0]);
		_delay_ms(100);
		set_digit(0);
		output_7segment(digits[1]);
		_delay_ms(100);
	}

	return 0;
}
